@@ -1,5 +1,6 @@
-from dexy.utils import os_to_posix
 from dexy.utils import copy_or_link
+from dexy.utils import os_to_posix
+from operator import attrgetter
 import dexy.doc
 import dexy.exceptions
 import dexy.plugin
@@ -38,6 +39,7 @@ class Filter(dexy.plugin.Plugin):
             'output-extensions' : ("List of extensions which this filter can produce as output.", [".*"]),
             'preserve-prior-data-class' : ('', False),
             'require-output' : ("Should dexy raise an exception if no output is produced by this filter?", True),
+            'tags' : ("Tags which describe the filter.", []),
             'variables' : ('', {}),
             'vars' : ('', {}),
             'workspace-exclude-filters' : ("Filters whose output should be excluded from workspace.", ['pyg'])
@@ -78,21 +80,30 @@ class Filter(dexy.plugin.Plugin):
 
         self.set_extension()
 
+        settings = self.input_data.setting_values()
+
+        del settings['storage-type']
+        del settings['aliases']
+        
+        settings.update({
+                'canonical-name' : self.calculate_canonical_name(),
+                'canonical-output' : self.is_canonical_output()
+                })
+
         self.output_data = dexy.data.Data.create_instance(
                 self.data_class_alias(self.ext),
                 self.key,
                 self.ext,
-                self.calculate_canonical_name(),
                 self.storage_key,
-                self.doc.setting_values(),
-                None,
-                self.is_canonical_output(),
+                settings,
                 self.doc.wrapper
                 )
 
     def is_canonical_output(self):
-        if self.input_data.canonical_output == True:
+        if self.input_data.setting('canonical-output') == True:
             return True
+        elif (self.input_data.setting('canonical-output') == False):
+            return False
         elif self.setting('output'):
             return True
         else:
@@ -110,7 +121,7 @@ class Filter(dexy.plugin.Plugin):
         # Check that we can handle input extension
         if set([prev_ext, ".*"]).isdisjoint(set(i_accept)):
             msg = "Filter '%s' in '%s' can't handle file extension %s, supported extensions are %s"
-            params = (self.filter_alias, self.key, prev_ext, ", ".join(i_accept))
+            params = (self.alias, self.key, prev_ext, ", ".join(i_accept))
             raise dexy.exceptions.UserFeedback(msg % params)
 
         # Figure out output extension
@@ -123,7 +134,7 @@ class Filter(dexy.plugin.Plugin):
             # Make sure it's a valid one
             if (not ext in i_output) and (not ".*" in i_output):
                 msg = "You have requested file extension %s in %s but filter %s can't generate that."
-                raise dexy.exceptions.UserFeedback(msg % (ext, self.key, self.filter_alias))
+                raise dexy.exceptions.UserFeedback(msg % (ext, self.key, self.alias))
 
             self.ext = ext
 
@@ -141,7 +152,7 @@ class Filter(dexy.plugin.Plugin):
                 else:
                     if set(i_output).isdisjoint(set(next_filter_accepts)):
                         msg = "Filter %s can't go after filter %s, no file extensions in common."
-                        raise dexy.exceptions.UserFeedback(msg % (self.next_filter_alias, self.filter_alias))
+                        raise dexy.exceptions.UserFeedback(msg % (self.next_filter.alias, self.alias))
 
                     for e in i_output:
                         if e in next_filter_accepts:
@@ -185,7 +196,7 @@ class Filter(dexy.plugin.Plugin):
         pass
 
     def calculate_canonical_name(self):
-        name_without_ext = posixpath.splitext(self.doc.canonical_name)[0]
+        name_without_ext = posixpath.splitext(self.doc.name)[0]
         return "%s%s" % (name_without_ext, self.ext)
 
     def output_filepath(self):
@@ -320,9 +331,7 @@ class Filter(dexy.plugin.Plugin):
 
         self._files_workspace_populated_with = set()
 
-        if os.path.exists(wd):
-            import shutil
-            shutil.rmtree(wd)
+        self.doc.wrapper.trash(wd)
 
         try:
             os.makedirs(wd)
@@ -358,8 +367,8 @@ class Filter(dexy.plugin.Plugin):
                 copy_or_link(data, file_dest)
 
             except Exception as e:
-                self.log_warn("problem populating working dir with input %s" % data.key)
-                self.log_warn(e)
+                self.log_debug("problem populating working dir with input %s" % data.key)
+                self.log_debug(e)
 
             self._files_workspace_populated_with.add(filepath)
 
@@ -404,27 +413,13 @@ class DexyFilter(Filter):
     """
     aliases = ['dexy']
 
-    def data_class_alias(self, file_ext):
-        if hasattr(self, 'process_dict'):
-            return 'sectioned'
-        elif hasattr(self, 'process_text_to_dict'):
-            return 'sectioned'
-        else:
-            return self.setting('output-data-type')
+    def data_class_alias(self, ext):
+        return self.setting('output-data-type')
 
     def process(self):
-        if hasattr(self, "process_text_to_dict"):
-            output = self.process_text_to_dict(unicode(self.input_data))
-            self.output_data.set_data(output)
-
-        elif hasattr(self, "process_dict"):
-            output = self.process_dict(self.input_data.as_sectioned())
-            self.output_data.set_data(output)
-
-        elif hasattr(self, "process_text"):
+        if hasattr(self, "process_text"):
             output = self.process_text(unicode(self.input_data))
             self.output_data.set_data(output)
-
         else:
             self.output_data.copy_from_file(self.input_data.storage.data_file())
 
@@ -439,3 +434,27 @@ class AliasFilter(DexyFilter):
 
     def calculate_canonical_name(self):
         return self.input_data.name
+
+def filters_by_tag():
+    """
+    Returns a dict with tags as keys and lists of corresponding filter instances as values.
+    """
+    tags_filters = {}
+    for filter_instance in Filter:
+        if filter_instance.setting('nodoc'):
+            continue
+
+        for tag in filter_instance.setting('tags'):
+            if not tags_filters.has_key(tag):
+                tags_filters[tag] = []
+            tags_filters[tag].append(filter_instance)
+
+    return tags_filters
+
+def filter_aliases_by_tag():
+    tags_filters = filters_by_tag()
+    tags = sorted(tags_filters.keys())
+    return [(tag,
+            [(filter_instance.alias, filter_instance.setting('help'))
+                for filter_instance in sorted(tags_filters[tag], key=attrgetter('alias'))])
+            for tag in tags]
