@@ -13,6 +13,8 @@ import logging.handlers
 import os
 import posixpath
 import shutil
+import sys
+import textwrap
 import time
 import uuid
 
@@ -21,7 +23,7 @@ class Wrapper(object):
     Class that manages run configuration and state and provides utilities such
     as logging and setting up/tearing down workspace on file system.
     """
-    _required_dirs = ['artifacts_dir', 'log_dir']
+    _required_dirs = ['artifacts_dir']
 
     state_transitions = (
             (None, 'new'),
@@ -34,6 +36,12 @@ class Wrapper(object):
             ('running', 'ran'),
             )
 
+    def printmsg(self, msg):
+        if self.silent:
+            self.log.warn(msg)
+        else:
+            print msg
+
     def validate_state(self, state=None):
         """
         Checks that the instance is in the expected state, and validates
@@ -62,6 +70,8 @@ class Wrapper(object):
         elif state == 'ran':
             for node in self.nodes.values():
                 assert node.state in ('ran', 'consolidated', 'inactive'), node.state
+        elif state == 'error':
+            pass
         else:
             raise dexy.exceptions.InternalDexyProblem(state)
 
@@ -72,6 +82,15 @@ class Wrapper(object):
         self.state = None
         self.transition('new')
 
+    def state_message(self):
+        """
+        A message to print at end of dexy run depending on the final wrapper state.
+        """
+        if self.state == 'error':
+            return " WITH ERRORS"
+        else:
+            return ""
+
     def transition(self, new_state):
         dexy.utils.transition(self, new_state)
 
@@ -152,30 +171,35 @@ class Wrapper(object):
             for d in hexes:
                 os.mkdir(os.path.join(cache_dir, "%s%s" % (c,d)))
 
-    def trash(self, d, d_exists=None):
+    def trash(self, d):
         """
         Move the passed file path (if it exists) into the .trash directory.
         """
-        if d_exists or os.path.exists(d):
-            if not self.is_location_in_project_dir(d):
-                msg = "trying to trash '%s', but this is not in project dir '%s"
-                msgargs = (d, self.project_root)
-                raise dexy.exceptions.InternalDexyProblem(msg % msgargs)
+        if not self.is_location_in_project_dir(d):
+            msg = "trying to trash '%s', but this is not in project dir '%s"
+            msgargs = (d, self.project_root)
+            raise dexy.exceptions.InternalDexyProblem(msg % msgargs)
 
-            trash_dir = self.trash_dir()
+        trash_dir = self.trash_dir()
 
-            if not os.path.exists(trash_dir):
-                os.mkdir(trash_dir)
+        try:
+            os.mkdir(trash_dir)
+        except OSError:
+            pass
 
-            move_to = os.path.join(trash_dir, str(uuid.uuid4()))
+        move_to = os.path.join(trash_dir, str(uuid.uuid4()))
+
+        try:
             shutil.move(d, move_to)
+        except IOError:
+            pass
 
     def empty_trash(self):
         try:
             shutil.rmtree(self.trash_dir())
         except OSError as e:
             if not "No such file or directory" in str(e):
-                raise e
+                raise
 
     def reset_work_cache_dir(self):
         # remove work/ dir leftover from previous run (if any) and create a new
@@ -191,34 +215,25 @@ class Wrapper(object):
 
         if self.target:
             matches = self.roots_matching_target()
+        else:
+            matches = self.roots
 
         try:
-            for node in self.roots:
-                if self.target:
-                    if not node in matches:
-                        continue
-
+            for node in matches:
                 for task in node:
+                    self.current_task = task
                     task()
 
-        except UserFeedback as e:
-            print e.message
-            self.error = e
-            self.transition('error')
-            if self.debug:
-                raise e
-        except InternalDexyProblem as e:
-            print e.message
-            self.error = e
-            self.transition('error')
-            if self.debug:
-                raise e
         except Exception as e:
-            print e
             self.error = e
             self.transition('error')
             if self.debug:
-                raise e
+                raise
+            else:
+                msg = "ERROR while running %s: %s\n" % (self.current_task.key, str(e))
+                sys.stderr.write(msg)
+                self.log.warn(msg)
+
         else:
             self.transition('ran')
             self.batch.end_time = time.time()
@@ -226,12 +241,32 @@ class Wrapper(object):
             shutil.move(self.this_cache_dir(), self.last_cache_dir())
             self.empty_trash()
 
+    def bundle_docs(self):
+        from dexy.node import BundleNode
+        return [n for n in self.nodes.values() if isinstance(n, BundleNode)]
+
+    def non_bundle_docs(self):
+        from dexy.node import BundleNode
+        return [n for n in self.nodes.values() if not isinstance(n, BundleNode)]
+
+    def documents(self):
+        from dexy.doc import Doc
+        return [n for n in self.nodes.values() if isinstance(n, Doc)]
+
     def roots_matching_target(self):
-        matches = [n for n in self.roots if n.key == self.target]
+        # First priority is to match any named bundles.
+        matches = [n for n in self.bundle_docs() if n.key == self.target]
         if not matches:
-            matches = [n for n in self.roots if n.key.startswith(self.target)]
+            # Second priority is exact matches of any document key.
+            matches = [n for n in self.non_bundle_docs() if n.key == self.target]
         if not matches:
+            # Third priority is partial matches of any document key.
             matches = [n for n in self.nodes.values() if n.key.startswith(self.target)]
+
+        if not matches:
+            raise dexy.exceptions.UserFeedback("No matches found for '%s'" % self.target)
+
+        self.log.debug("Documents matching target '%s' are: %s" % (self.target, ", ".join(m.key_with_class() for m in matches)))
         return matches
 
     def run_from_new(self):
@@ -239,7 +274,7 @@ class Wrapper(object):
         self.to_walked()
         self.to_checked()
         if self.dry_run:
-            print "dry run only"
+            self.printmsg("dry run only")
         else:
             self.run()
 
@@ -343,10 +378,15 @@ class Wrapper(object):
         Raise a UserFeedback error if user has tried to run dexy without
         setting up necessary directories first.
         """
+        self.detect_dot_dexy_files()
+        self.update_cache_directory()
+
         if not self.dexy_dirs_exist():
             msg = "You need to run 'dexy setup' in this directory first."
             raise UserFeedback(msg)
 
+        self.deprecate_logs_directory()
+
     def create_dexy_dirs(self):
         """
         Creates the directories needed for dexy to run. Does not complain if
@@ -358,10 +398,81 @@ class Wrapper(object):
                 with open(safety_filepath, 'w') as f:
                     f.write("This directory was created by dexy.")
 
+    def detect_dot_dexy_files(self):
+        if os.path.exists(".dexy") and os.path.isfile(".dexy"):
+            msg = """\
+            You have a file named '.dexy' in your project, this format is no longer supported.
+            See http://dexy.it/guide/getting-started.html for updated tutorials.
+            Please rename or remove this file to proceed."""
+            raise dexy.exceptions.UserFeedback(textwrap.dedent(msg))
+
+    def update_cache_directory(self):
+        """
+        Move .cache directories to be .dexy directories.
+        """
+        old_cache_dir = ".cache"
+        safety_file = os.path.join(old_cache_dir, self.safety_filename)
+
+        if self.artifacts_dir == old_cache_dir:
+            msg = """\
+            You may have a dexy.conf file present in which you specify
+            'artifactsdir: .cache'
+            
+            Dexy's defaults have changed and the .cache directory is being
+            renamed to .dexy
+
+            The easiest way to never see this message again is to remove this
+            line from your dexy.conf file (You should remove any entries in
+            dexy.conf which you don't specifically want to customize.)
+            
+            If you really want the artifacts directory to be at .cache then you
+            can leave your config in place and this message will disappear in a
+            future dexy version.
+            """
+            print textwrap.dedent(msg)
+
+        elif os.path.exists(old_cache_dir) and os.path.isdir(old_cache_dir) and os.path.exists(safety_file):
+            if os.path.exists(self.artifacts_dir):
+                if os.path.isdir(self.artifacts_dir):
+                    msg = "You have a dexy '%s' directory and a '%s' directory. Please remove '%s' or at least '%s'."
+                    msgargs = (old_cache_dir, self.artifacts_dir, old_cache_dir, safety_file)
+                    raise dexy.exceptions.UserFeedback(msg % msgargs)
+                else:
+                    msg = "'%s' is not a dir!" % (self.artifacts_dir,)
+                    raise dexy.exceptions.InternalDexyProblem(msg)
+
+            print "Moving directory '%s' to new location '%s'" % (old_cache_dir, self.artifacts_dir)
+            shutil.move(old_cache_dir, self.artifacts_dir)
+
+    def deprecate_logs_directory(self):
+        log_dir = 'logs'
+        if self.log_dir != self.artifacts_dir:
+            # user has set a custom log dir
+            log_dir = self.log_dir
+
+        safety_file = os.path.join(log_dir, self.safety_filename)
+        deprecation_notice_file = os.path.join(log_dir, "WHERE-ARE-THE-LOGS.txt")
+
+        if os.path.exists(log_dir) and os.path.exists(safety_file) and not os.path.exists(deprecation_notice_file):
+            deprecation_notice = """\
+            Dexy no longer has a separate '{log_dir}' directory for the log file.
+            The logfile can now be found at: {log_path}\n""".format(
+                    log_path = self.log_path(),
+                    log_dir = log_dir)
+            
+            deprecation_notice = "\n".join(l.strip() for l in deprecation_notice.splitlines())
+            self.printmsg("Deprecating %s/ directory" % log_dir)
+            self.printmsg(deprecation_notice)
+            self.printmsg("You can remove the %s/ directory" % log_dir)
+
+            with open(deprecation_notice_file, 'w') as f:
+                f.write(deprecation_notice + "\n\nYou can remove this directory.\n")
+            self.trash(os.path.join(log_dir, "dexy.log"))
+
     def remove_dexy_dirs(self):
         for dirpath, safety_filepath, dirstat in self.iter_dexy_dirs():
             if dirstat:
-                self.trash(dirpath, True)
+                self.trash(dirpath)
         self.empty_trash()
 
     def remove_reports_dirs(self, reports=True, keep_empty_dir=False):
@@ -371,14 +482,14 @@ class Wrapper(object):
                 reports=dexy.reporter.Reporter
 
             for report in reports:
-                report.remove_reports_dir(keep_empty_dir)
+                report.remove_reports_dir(self, keep_empty_dir)
 
     # Logging
     def log_path(self):
         """
         Returns path to logfile.
         """
-        return os.path.join(self.log_dir, self.log_file)
+        return os.path.join(self.artifacts_dir, self.log_file)
 
     def setup_log(self):
         """
@@ -397,6 +508,10 @@ class Wrapper(object):
         self.log.setLevel(log_level)
         self.log.addHandler(handler)
         self.log.info("starting logging for dexy")
+        self.log_handler = handler
+
+    def flush_logs(self):
+        self.log_handler.flush()
 
     # Project files
     def exclude_dirs(self):
@@ -431,7 +546,7 @@ class Wrapper(object):
 
         for dirpath, dirnames, filenames in os.walk('.'):
             for x in exclude:
-                if x in dirnames:
+                if x in dirnames and not x in self.include:
                     dirnames.remove(x)
 
             if '.nodexy' in filenames:
@@ -554,7 +669,7 @@ class Wrapper(object):
         # warn if we don't find any configs
         if len(config_files) == 0:
             msg = "didn't find any document config files (like %s)"
-            print msg % (", ".join(parser_aliases))
+            self.printmsg(msg % (", ".join(parser_aliases)))
 
         # parse each config file and add to ast
         ast = dexy.parser.AbstractSyntaxTree(self)
@@ -585,4 +700,4 @@ class Wrapper(object):
                 reporter.run(self)
 
     def is_location_in_project_dir(self, filepath):
-        return os.path.abspath(self.project_root) in os.path.abspath(filepath)
+        return self.project_root in os.path.abspath(filepath)
